@@ -37,6 +37,7 @@ class AddressGroupDbMixin(ag_ext.AddressGroupPluginBase):
         res = address_group.to_dict()
         res['addresses'] = [str(addr_assoc['address'])
                             for addr_assoc in address_group['addresses']]
+        res['standard_attr_id'] = address_group.standard_attr_id
         return db_utils.resource_fields(res, fields)
 
     @staticmethod
@@ -100,7 +101,8 @@ class AddressGroupDbMixin(ag_ext.AddressGroupPluginBase):
                          payload=events.DBEventPayload(
                              context,
                              resource_id=address_group_id,
-                             states=(original_address_group, ag_dict,)))
+                             states=(original_address_group,
+                                     ag_dict['address_group'],)))
         return ag_dict
 
     def remove_addresses(self, context, address_group_id, addresses):
@@ -120,7 +122,8 @@ class AddressGroupDbMixin(ag_ext.AddressGroupPluginBase):
                          payload=events.DBEventPayload(
                              context,
                              resource_id=address_group_id,
-                             states=(original_address_group, ag_dict,)))
+                             states=(original_address_group,
+                                     ag_dict['address_group'],)))
         return ag_dict
 
     def create_address_group(self, context, address_group):
@@ -166,6 +169,7 @@ class AddressGroupDbMixin(ag_ext.AddressGroupPluginBase):
     def get_address_groups(self, context, filters=None, fields=None,
                            sorts=None, limit=None, marker=None,
                            page_reverse=False):
+        filters = filters or {}
         pager = base_obj.Pager(sorts, limit, page_reverse, marker)
         address_groups = ag_obj.AddressGroup.get_objects(
             context, _pager=pager, **filters)
