@@ -35,6 +35,7 @@ from neutron.objects.port_forwarding import PortForwarding
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb.extensions import qos \
     as ovn_qos
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovn_client
+from neutron.services.logapi.drivers.ovn import driver as log_driver
 from neutron.services.segments import db as segments_db
 
 
@@ -86,6 +87,18 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
             self.segments_plugin = (
                 manager.NeutronManager.load_class_for_provider(
                     'neutron.service_plugins', 'segments')())
+        self.log_plugin = directory.get_plugin(plugin_constants.LOG_API)
+        if not self.log_plugin:
+            self.log_plugin = (
+                manager.NeutronManager.load_class_for_provider(
+                    'neutron.service_plugins', 'log')())
+            directory.add_plugin(plugin_constants.LOG_API, self.log_plugin)
+        for driver in self.log_plugin.driver_manager.drivers:
+            if driver.name == "ovn":
+                self.ovn_log_driver = driver
+        if not hasattr(self, 'ovn_log_driver'):
+            self.ovn_log_driver = log_driver.OVNDriver()
+            self.log_plugin.driver_manager.register_driver(self.ovn_log_driver)
 
     def stop(self):
         if utils.is_ovn_l3(self.l3_plugin):
@@ -236,15 +249,30 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
 
     def _get_acls_from_port_groups(self):
         ovn_acls = []
+        # Options and label columns are only present for OVN >= 22.03.
+        # Furthermore label is a randint so it cannot be compared with any
+        # expected neutron value. They are added later on the ACL addition.
         acl_columns = (self.ovn_api._tables['ACL'].columns.keys() &
                        set(ovn_const.ACL_EXPECTED_COLUMNS_NBDB))
         acl_columns.discard('external_ids')
+        id_key = ovn_const.OVN_SG_RULE_EXT_ID_KEY
         for pg in self.ovn_api.db_list_rows('Port_Group').execute():
             acls = getattr(pg, 'acls', [])
             for acl in acls:
                 acl_string = {k: getattr(acl, k) for k in acl_columns}
                 acl_string['port_group'] = pg.name
+                if id_key in acl.external_ids:
+                    acl_string[id_key] = acl.external_ids[id_key]
+                # This properties are present as lists of one item,
+                # converting them to string.
+                if acl_string['name']:
+                    acl_string['name'] = acl_string['name'][0]
+                if acl_string['meter']:
+                    acl_string['meter'] = acl_string['meter'][0]
+                if acl_string['severity']:
+                    acl_string['severity'] = acl_string['severity'][0]
                 ovn_acls.append(acl_string)
+
         return ovn_acls
 
     def sync_acls(self, ctx):
@@ -275,10 +303,74 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                 pg_name = utils.ovn_port_group_name(sgr['security_group_id'])
                 neutron_acls.append(acl_utils._add_sg_rule_acl_for_port_group(
                     pg_name, True, sgr))
-        neutron_acls += acl_utils.add_acls_for_drop_port_group(
+        # Sort the acls in the Neutron database according to the security
+        # group rule ID for easy comparison in the future.
+        neutron_acls.sort(key=lambda x: x[ovn_const.OVN_SG_RULE_EXT_ID_KEY])
+        neutron_default_acls = acl_utils.add_acls_for_drop_port_group(
             ovn_const.OVN_DROP_PORT_GROUP_NAME)
 
+        # Add logging options
+        self.ovn_log_driver.add_logging_options_to_acls(neutron_acls, ctx)
+        self.ovn_log_driver.add_logging_options_to_acls(neutron_default_acls,
+                                                        ctx)
         ovn_acls = self._get_acls_from_port_groups()
+        # Sort the acls in the ovn database according to the security
+        # group rule id for easy comparison in the future.
+        ovn_acls.sort(key=lambda x: x.get(
+            ovn_const.OVN_SG_RULE_EXT_ID_KEY, ""))
+        neutron_num, ovn_num = len(neutron_acls), len(ovn_acls)
+        add_acls, remove_acls, ovn_default_acls = [], [], []
+        n_index = o_index = 0
+        # neutron_acls and ovn_acls have been sorted, and we need to traverse
+        # both arrays from scratch until we reach the end of one of them.
+        # If it is found that the elements of two arrays are consistent,
+        # it is considered that the data is consistent. If it is found during
+        # comparison that the security group rule id in neutron_acls is
+        # smaller than the security group rule id in ovn_acls, as the arrays
+        # have been sorted, this security group rule will no longer appear in
+        # the future. This indicates that this security group rule only
+        # exists in neutron_acls. If it is a related situation, this security
+        # group rule only appears in ovn_acls.
+        while n_index < neutron_num and o_index < ovn_num:
+            na, oa = neutron_acls[n_index], ovn_acls[o_index]
+            n_id = na[ovn_const.OVN_SG_RULE_EXT_ID_KEY]
+            o_id = oa.get(ovn_const.OVN_SG_RULE_EXT_ID_KEY)
+            if not o_id:
+                # There may be some ACLs in the OVN database that do not have
+                # security group rule id. These are the default rules, which
+                # will be specially compared and processed later.
+                ovn_default_acls.append(oa)
+                o_index += 1
+            elif n_id == o_id:
+                if any(item not in na.items() for item in oa.items()):
+                    for item in oa.items():
+                        if item not in na.items():
+                            LOG.warning('Property %(item)s from OVN ACL not '
+                                        'found in Neutron ACL: %(n_acl)s',
+                                        {'item': item,
+                                         'n_acl': na})
+                    add_acls.append(na)
+                    remove_acls.append(oa)
+                n_index += 1
+                o_index += 1
+            elif n_id > o_id:
+                LOG.warning('ACL should not be present in OVN, removing'
+                            '%(acl)s', {'acl': oa})
+                remove_acls.append(oa)
+                o_index += 1
+            else:
+                LOG.warning('ACL should be present in OVN but is not, adding:'
+                            '%(acl)s', {'acl': na})
+                add_acls.append(na)
+                n_index += 1
+
+        if n_index < neutron_num:
+            # We didn't find the OVN ACLs matching the Neutron ACLs
+            # in "ovn_acls" and we are just adding the pending Neutron ACLs.
+            add_acls.extend(neutron_acls[n_index:])
+        if o_index < ovn_num:
+            # Any OVN ACLs not matching the Neutron ACLs is removed.
+            remove_acls.extend(ovn_acls[o_index:])
 
         # We need to remove also all the ACLs applied to Logical Switches
         def get_num_acls(ovn_acls):
@@ -289,13 +381,14 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         num_acls_to_remove_from_ls = get_num_acls(ovn_acls_from_ls)
 
         # Remove the common ones
-        for na in list(neutron_acls):
-            for ovn_a in ovn_acls.copy():
+        for na in list(neutron_default_acls):
+            for ovn_a in ovn_default_acls.copy():
                 if all(item in na.items() for item in ovn_a.items()):
-                    neutron_acls.remove(na)
-                    ovn_acls.remove(ovn_a)
+                    neutron_default_acls.remove(na)
+                    ovn_default_acls.remove(ovn_a)
                     break
-
+        neutron_acls = add_acls + neutron_default_acls
+        ovn_acls = remove_acls + ovn_default_acls
         num_acls_to_add = len(neutron_acls)
         num_acls_to_remove = len(ovn_acls) + num_acls_to_remove_from_ls
         if num_acls_to_add or num_acls_to_remove:
@@ -307,32 +400,6 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         if (self.mode == SYNC_MODE_REPAIR and
                 (num_acls_to_add or num_acls_to_remove)):
             one_time_pg_resync = True
-            while True:
-                try:
-                    with self.ovn_api.transaction(check_error=True) as txn:
-                        for acla in neutron_acls:
-                            LOG.warning('ACL found in Neutron but not in '
-                                        'OVN NB DB for port group %s',
-                                        acla['port_group'])
-                            txn.add(self.ovn_api.pg_acl_add(
-                                **acla, may_exist=True))
-                except idlutils.RowNotFound as row_err:
-                    if row_err.msg.startswith("Cannot find Port_Group"):
-                        if one_time_pg_resync:
-                            LOG.warning('Port group row was not found during '
-                                        'ACLs sync. Will attempt to sync port '
-                                        'groups one more time. The caught '
-                                        'exception is: %s', row_err)
-                            self.sync_port_groups(ctx)
-                            one_time_pg_resync = False
-                            continue
-                        LOG.error('Port group exception during ACL sync '
-                                  'even after one more port group resync. '
-                                  'The caught exception is: %s', row_err)
-                    else:
-                        raise
-                break
-
             with self.ovn_api.transaction(check_error=True) as txn:
                 for aclr in ovn_acls:
                     LOG.warning('ACLs found in OVN NB DB but not in '
@@ -349,6 +416,41 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                         LOG.warning('Removing ACLs from OVN from Logical '
                                     'Switch %s', aclr[0])
                         txn.add(self.ovn_api.acl_del(aclr[0]))
+            while True:
+                try:
+                    with self.ovn_api.transaction(check_error=True) as txn:
+                        for acla in neutron_acls:
+                            LOG.warning('ACL found in Neutron but not in '
+                                        'OVN NB DB for port group %s',
+                                        acla['port_group'])
+                            acl = txn.add(self.ovn_api.pg_acl_add(**acla,
+                                                               may_exist=True))
+                            # We need to do this now since label should be
+                            # random and not 0. We can use options as a way
+                            # to see if label is supported or not.
+                            if acla.get('log'):
+                                self.ovn_log_driver.add_label_related(acla,
+                                                                      ctx)
+                                txn.add(self.ovn_api.db_set('ACL', acl,
+                                    label=acla['label'],
+                                    options=acla['options']))
+
+                except idlutils.RowNotFound as row_err:
+                    if row_err.msg.startswith("Cannot find Port_Group"):
+                        if one_time_pg_resync:
+                            LOG.warning('Port group row was not found during '
+                                        'ACLs sync. Will attempt to sync port '
+                                        'groups one more time. The caught '
+                                        'exception is: %s', row_err)
+                            self.sync_port_groups(ctx)
+                            one_time_pg_resync = False
+                            continue
+                        LOG.error('Port group exception during ACL sync '
+                                  'even after one more port group resync. '
+                                  'The caught exception is: %s', row_err)
+                    else:
+                        raise
+                break
 
         LOG.debug('OVN-NB Sync ACLs completed @ %s', str(datetime.now()))
 
